@@ -8,8 +8,10 @@ from abc import ABCMeta, abstractmethod
 from pathlib import Path
 import asyncio
 
-from codelab_adapter_client.topic import ADAPTER_TOPIC, SCRATCH_TOPIC, NOTIFICATION_TOPIC, EXTENSIONS_OPERATE_TOPIC
+from codelab_adapter_client.topic import *
+# from codelab_adapter_client.topic import ADAPTER_TOPIC, SCRATCH_TOPIC, NOTIFICATION_TOPIC, EXTENSIONS_OPERATE_TOPIC
 from codelab_adapter_client.utils import threaded
+from codelab_adapter_client.session import _message_template
 
 logger = logging.getLogger(__name__)
 
@@ -22,7 +24,7 @@ class MessageNodeAio(metaclass=ABCMeta):
             codelab_adapter_ip_address=None,
             subscriber_port='16103',
             publisher_port='16130',
-            subscriber_list=None,
+            subscriber_list=[SCRATCH_TOPIC, NODES_OPERATE_TOPIC],
             loop_time=0.1,
             connect_time=0.3,
             external_message_processor=None,
@@ -44,6 +46,7 @@ class MessageNodeAio(metaclass=ABCMeta):
             self.name = name
         else:
             self.name = type(self).__name__
+        self.token = token
         self.subscriber_list = subscriber_list
         self.receive_loop_idle_addition = receive_loop_idle_addition
         self.external_message_processor = external_message_processor
@@ -91,7 +94,7 @@ class MessageNodeAio(metaclass=ABCMeta):
 
         if not type(topic) is str:
             raise TypeError('Subscriber topic must be string')
-
+        # todo: base.py
         self.subscriber.setsockopt(zmq.SUBSCRIBE, topic.encode())
 
     async def pack(self, data):
@@ -138,10 +141,15 @@ class MessageNodeAio(metaclass=ABCMeta):
                 await self.set_subscriber_topic(topic)
 
         while self._running:
-            data = await self.subscriber.recv_multipart()
-            topic = data[0].decode()
-            payload = await self.unpack(data[1])
-            await self.message_handle(topic, payload)
+            # NOBLOCK
+            try:
+                data = await self.subscriber.recv_multipart(zmq.NOBLOCK)
+                # data = await self.subscriber.recv_multipart()
+                topic = data[0].decode()
+                payload = await self.unpack(data[1])
+                await self.message_handle(topic, payload)
+            except zmq.error.Again:
+                await asyncio.sleep(self.loop_time)
 
     async def start_the_receive_loop(self):
         self.receive_loop_task = self.event_loop.create_task(
@@ -161,10 +169,13 @@ class MessageNodeAio(metaclass=ABCMeta):
         """
         Clean up before exiting.
         """
+
         self._running = False
-        await self.publisher.close()
-        await self.subscriber.close()
-        await self.context.term()
+        await asyncio.sleep(0.1)
+        # await self.publisher.close()
+        # await self.subscriber.close()
+        # await self.context.term()
+        
 
 
 class AdapterNodeAio(MessageNodeAio):
@@ -186,4 +197,53 @@ class AdapterNodeAio(MessageNodeAio):
         self.ADAPTER_TOPIC = ADAPTER_TOPIC  # message topic: the message from adapter
         self.SCRATCH_TOPIC = SCRATCH_TOPIC  # message topic: the message from scratch
         if not hasattr(self, 'EXTENSION_ID'):
-            self.EXTENSION_ID = "eim"
\ No newline at end of file
+            self.EXTENSION_ID = "eim"
+        if not hasattr(self, 'TOPIC'):
+            self.TOPIC = ADAPTER_TOPIC  # message topic: the message from adapter
+
+
+    def message_template(self):
+        # _message_template(sender,username,extension_id,token) dict
+        template = _message_template(self.name, self.EXTENSION_ID, self.token)
+        return template
+    
+    async def publish(self, message):
+        assert isinstance(message, dict)
+        topic = message.get('topic')
+        payload = message.get("payload")
+        if not topic:
+            topic = self.TOPIC
+        if not payload.get("extension_id"):
+            payload["extension_id"] = self.EXTENSION_ID
+        self.logger.debug(
+            f"{self.name} publish: topic: {topic} payload:{payload}")
+
+        await self.publish_payload(payload, topic)
+
+    async def pub_notification(self, content, topic=NOTIFICATION_TOPIC, type="INFO"):
+        '''
+        type
+            ERROR
+            INFO
+        {
+            topic: "from_adapter/extensions/notification"
+            payload: {
+                content:
+            }
+        }
+        '''
+        extension_id = self.EXTENSION_ID
+        payload = self.message_template()["payload"]
+        payload["type"] = type
+        payload["content"] = content
+        await self.publish_payload(payload, topic)
+    
+    def terminate(self):
+        '''
+        stop by thread
+        await 
+        '''
+        # await self.clean_up() # todo 同步中运行异步
+
+        self.event_loop.create_task(self.clean_up())
+        self.logger.info(f"{self} terminate!")
\ No newline at end of file
