@@ -1,3 +1,19 @@
+'''
+channel
+
+将RPC和 [CQRS](https://en.wikipedia.org/wiki/Command%E2%80%93query_separation) 都放在pubsub里 (ROS)
+
+
+[jupyter client messaging](https://jupyter-client.readthedocs.io/en/stable/messaging.html)
+    *  Shell code execution req-rep
+    *  Jupyter kernel Control channel
+        *  shutdown/restart
+    *  stdin: input from user
+    *  IOPub
+        *  stdout, stderr, debugging events
+'''
+
+# notification
 # adapter data
 ADAPTER_TOPIC = "adapter/extensions/data"  # 来自插件的消息，topic
 
@@ -15,17 +31,18 @@ EXTENSIONS_STATUS_TRIGGER_TOPIC = "core/extensions/status/trigger"
 EXTENSION_STATU_CHANGE_TOPIC = "core/extension/statu/change"
 NOTIFICATION_TOPIC = "core/notification"
 GUI_TOPIC = "gui/operate"
+
 # ble
 ADAPTER_BLE_TOPIC = "adapter/ble"
 SCRATCH_BLE_TOPIC = "scratch/ble"
 
-# mqtt
+# mqtt gateway(from/to)
 TO_MQTT_TOPIC = "to_mqtt"
 FROM_MQTT_TOPIC = "from_mqtt"
 
 # jupyter
 
-# Home Assistant
+# Home Assistant gateway(from/to)
 FROM_HA_TOPIC = "from_HA"
 TO_HA_TOPIC = "to_HA"
 
